-- tstDE0.vhd	ajm						07-jun-2017
--
-- entity	tstDE0
--			-top-level testbench, entity:	clock
--			-			-"-	dcf77
-- architecture	stimuli
--			-the stimuli generation is for the sample push-button
--			 interface: set_time/set_ala + set_mins/set_hrs
------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

entity tstDE0 is
generic (	tClk	: time		:= 20 ns);	-- 50 MHz
end entity tstDE0;

------------------------------------------------------------------------------
architecture stimuli of tstDE0 is

  -- component decl.	------------------------------------------------------
  ----------------------------------------------------------------------------
  component de0Board is
  port(	clk50		: in  std_logic;	-- 50 MHz clock		R
	dramCsN		: out std_logic;	-- PCB hardware
	epcsCsN		: out std_logic;
	adcCsN		: out std_logic;
	reset		: in  std_logic;	-- async. reset		L
	dcfsig		: in  std_logic;	-- DCF77 signal
	set_ala		: in  std_logic;	-- Set-Alarm  button	H
	set_time	: in  std_logic;	-- Set-Time   button	H
	set_hrs		: in  std_logic;	-- Set-Hour   button	H
	set_mins	: in  std_logic;	-- Set-Min    button	H
	disp_date	: in  std_logic;	-- date       button	H
	disp_wday	: in  std_logic;	-- weekday    button	H
	alarm_tog	: in  std_logic;	-- alarm      button	H
	seldgt		: out std_logic_vector (5 downto 0);	--digit lines	
	decoded		: out std_logic_vector (7 downto 0);	--7 Seg output
	alarm_act	: out std_logic;	-- alarm LED on		H
	alarm_out	: out std_logic;	-- alarm ringer		H
	dcfsok		: out std_logic;	-- signal quality	H
	dcfout		: out std_logic);	-- debug signal
  end component de0Board;

  ----------------------------------------------------------------------------
  component dcf77 is
  generic(	tDelay	: time		:= 300 us;	-- initial Delay
		tSample	: time		:=   1 ms);	-- 1 kHz sample freq.
  port(		dcfsig	: out std_logic;		-- DCF77 signal
		mins	: in  integer range 0 to 59	:= 34;
		hours	: in  integer range 0 to 23	:= 12;
		wday	: in  integer range 1 to  7	:= 1;
		day	: in  integer range 1 to 31	:= 25;
		month	: in  integer range 1 to 12	:= 11;
		year	: in  integer range 0 to 99	:= 2;
		noise	: in  integer range 0 to 50	:= 0;
		xtra	: in  std_logic_vector (15 to 19):= "00000");
  end component dcf77;

  -- signal decl.	------------------------------------------------------
  ----------------------------------------------------------------------------
  signal reset		: std_logic := '0';
  signal clk50		: std_logic := '0';
  signal dcfSig, dcfsN	: std_logic;
  signal butTime	: std_logic := '0';
  signal butAla		: std_logic := '0';
  signal butMins	: std_logic := '0';
  signal butHrs		: std_logic := '0';
  signal butAlarm	: std_logic := '0';
  signal butDate	: std_logic := '0';
  signal butWDay	: std_logic := '0';
  signal seldgt		: std_logic_vector (5 downto 0);
  signal decoded	: std_logic_vector (7 downto 0);
  signal alarmLed	: std_logic;
  signal alarmRing	: std_logic;
  signal dcfsok		: std_logic;
  signal dcfout		: std_logic;
  signal noise		: integer range 0 to 50	:= 0;

begin
  -- instances		------------------------------------------------------
  ----------------------------------------------------------------------------
  clockI: de0Board
    port map(	clk50		=>	clk50,
		dramCsN		=>	open,
		epcsCsN		=>	open,
		adcCsN		=>	open,
		reset		=>	reset,
		dcfsig		=>	dcfsN,	--not dcfSig,
		set_ala		=>	butAla,
		set_time	=>	butTime,
		set_hrs		=>	butHrs,
		set_mins	=>	butMins,
		disp_date	=>	butDate,
		disp_wday	=>	butWDay,
		alarm_tog	=>	butAlarm,
		seldgt		=>	seldgt,
		decoded		=>	decoded,
		alarm_act	=>	alarmLed,
		alarm_out	=>	alarmRing,
		dcfsok		=>	dcfsok,
		dcfout		=>	dcfout);

  -- default time: 12:34, Mo. 25.11.02		------------------------------
  dcf77I: dcf77
    generic map	(tDelay => 1587654 us)		-- async. start, long pause
    port map	(dcfsig	=> dcfSig,
		 noise	=> noise);

  dcfsN <= not dcfSig;

  -- processes		------------------------------------------------------
  ----------------------------------------------------------------------------

  -- clock, period: tClk 			------------------------------
  clkP: process
  begin
	clk50 <= '0';	wait for tClk/2;	-- 10 ns
	clk50 <= '1';	wait for tClk/2;	-- 10 ns
  end process clkP;
 
  -- reset, trigger				------------------------------
  rstP: process
  begin
	reset <= '1';	wait for tClk/10;	--  2 ns
	reset <= '0';	wait for tClk/5;	--  4 ns
	reset <= '1';	wait for tClk/2;	-- 10 ns
	wait on reset;
  end process rstP;

  -- stimuli generator				------------------------------
  stiP: process
    -- local (overloaded) procedures		------------------------------
    procedure pushBut(	pushT, runT		: in time;
			signal but		: out std_logic) is
    begin
      but <= '1';	wait for pushT;
      but <= '0';	wait for runT-pushT;
    end procedure pushBut;			-- one button

    procedure pushBut(	pushT, runT		: in time;
			signal but1, but2	: out std_logic) is
    begin
      but1 <= '1';
      but2 <= '1';	wait for pushT;
      but1 <= '0';
      but2 <= '0';	wait for runT-pushT;
    end procedure pushBut;			-- two buttons
  begin
    -- send pattern:	12:34, Mo. 25.11.02	-- <action>    <time-Sum>
    wait for 1 sec;				--		 1

    -- set alarm: 	12:34
    pushBut( 6 sec,  7 sec, butAla, butHrs);	-- hour   +12	 8
    pushBut(17 sec, 18 sec, butAla, butMins);	-- minute +34	26

    -- activate alarm
    pushBut(300 ms, 1 sec, butAlarm);		-- alarm on	27

    -- set time: 	06:08
    pushBut( 3 sec,  4 sec, butTime, butHrs);	-- hour   + 3	31
    pushBut( 4 sec,  5 sec, butTime, butMins);	-- minute + 4	36

    -- run for 28 sec., synchronisation at tDelay + 60 sec. =>  62
    wait for 28 sec;				--		(64) -> 02

    -- stop alarm
    pushBut(300 ms, 1 sec, butAlarm);		-- alarm off	(65) -> 03

    wait;
  end process stiP;

end architecture stimuli;
